#include "mvbluefox3_settings.h"
#include "exceptions/mvbluefox3_exceptions.h"
#include <mvIMPACT_CPP/mvIMPACT_acquire.h>
#include <iostream>


namespace CMvbluefox3 {

codings_t MVFormatToEncoding(const TImageDestinationPixelFormat& pixel_format) 
{
  codings_t coding;

  switch (pixel_format) 
  {
    case idpfAuto : coding = raw; break;  //*
    case idpfRaw: coding = raw; break;
    case idpfMono8: coding = mono8; break;
    case idpfMono10: coding = mono10; break; //* 
    case idpfMono12: coding = mono12; break; //*  
    case idpfMono12Packed_V1: coding = mono12v1; break; //*  
    case idpfMono12Packed_V2: coding = mono12v2; break; //*  
    case idpfMono14: coding = mono14; break; //*            
    case idpfMono16: coding = mono16; break;  
    case idpfBGR888Packed: coding = rgb8; break;
    case idpfRGB888Planar: coding = bgr8p; break; //*  
    case idpfRGBx888Planar: coding = bgra8p; break; //*  
    case idpfRGB888Packed: coding = bgr8; break;
    case idpfRGB101010Packed: coding = bgr10; break; //*
    case idpfRGB121212Packed: coding = bgr12; break; //*
    case idpfRGB141414Packed: coding = bgr14; break; //*
    case idpfRGB161616Packed: coding = bgr16; break;
    case idpfRGBx888Packed: coding = bgra8; break;
    case idpfYUV411_UYYVYY_Packed: coding = yuv411uyyvyy; break; //*
    case idpfYUV422Packed: coding = yuv422; break;  
    case idpfYUV422Planar: coding = yuv422p; break; //*
    case idpfYUV422_10Packed: coding = yuv42210; break; //*  
    case idpfYUV422_UYVYPacked: coding = yuv422uyvy; break; //*  
    case idpfYUV422_UYVY_10Packed: coding = yuv422uyvy10; break; //*  
    case idpfYUV444Packed: coding = yuv444; break; //*
    case idpfYUV444_10Packed: coding = yuv44410p; break; //*
    case idpfYUV444_UYVPacked: coding = yuv444uyv; break; //*
    case idpfYUV444_UYV_10Packed: coding = yuv444uyv10; break; //*
  }
  return coding;
}

TImageDestinationPixelFormat EncodingToMVFormat(const codings_t& coding)
{
  TImageDestinationPixelFormat idpf;
  switch (coding)
  {
    default: idpf = idpfAuto; break;  //*
    case raw: idpf = idpfRaw; break;
    case mono8: idpf = idpfMono8; break;
    case mono10: idpf = idpfMono10; break; //* 
    case mono12: idpf = idpfMono12; break; //*  
    case mono12v1: idpf = idpfMono12Packed_V1; break; //*  
    case mono12v2: idpf = idpfMono12Packed_V2; break; //*  
    case mono14: idpf = idpfMono14; break; //*            
    case mono16: idpf = idpfMono16; break;  
    case rgb8: idpf = idpfBGR888Packed; break;
    case bgr8p: idpf = idpfRGB888Planar; break; //*  
    case bgra8p: idpf = idpfRGBx888Planar; break; //*  
    case rgb10v2: idpf = idpfBGR101010Packed_V2; break; //*
    case bgr8: idpf = idpfRGB888Packed; break;
    case bgr10: idpf = idpfRGB101010Packed; break; //*
    case bgr12: idpf = idpfRGB121212Packed; break; //*
    case bgr14: idpf = idpfRGB141414Packed; break; //*
    case bgr16: idpf = idpfRGB161616Packed; break;
    case bgra8: idpf = idpfRGBx888Packed; break;
    case yuv411uyyvyy: idpf = idpfYUV411_UYYVYY_Packed; break; //*
    case yuv422: idpf = idpfYUV422Packed; break;  
    case yuv422p: idpf = idpfYUV422Planar; break; //*
    case yuv42210: idpf = idpfYUV422_10Packed; break; //*  
    case yuv422uyvy: idpf = idpfYUV422_UYVYPacked; break; //*  
    case yuv422uyvy10: idpf = idpfYUV422_UYVY_10Packed; break; //*  
    case yuv444: idpf = idpfYUV444Packed; break; //*
    case yuv44410p: idpf = idpfYUV444_10Packed; break; //*
    case yuv444uyv: idpf = idpfYUV444_UYVPacked; break; //*
    case yuv444uyv10: idpf = idpfYUV444_UYV_10Packed; break; //*
  }
  return idpf;
}

codings_t BayerPatternToEncoding(const TBayerMosaicParity& bayer_pattern,
                                  int bytes_per_pixel) 
{
  codings_t coding;
  if (bytes_per_pixel == 1) 
  {
    switch (bayer_pattern) 
    {
      case bmpRG: coding = bayer_rggb8; break;
      case bmpGB: coding = bayer_gbrg8; break;
      case bmpGR: coding = bayer_grbg8; break;
      case bmpBG: coding = bayer_bggr8; break;
      default: coding = mono8; break;
    }
  } 
  else if (bytes_per_pixel == 2) 
  {
    switch (bayer_pattern) 
    {
      case bmpRG: coding = bayer_rggb16; break;
      case bmpGB: coding = bayer_gbrg16; break;
      case bmpGR: coding = bayer_grbg16; break;
      case bmpBG: coding = bayer_bggr16; break;
      default: coding = mono16; break;
    }
  }
  return coding;
}

bayer_filter_t MVFormatToColorFilter(const int &pcf)
{
  bayer_filter_t bf;
  switch (pcf)
  {
    case 0: bf = none; break;
    case 1: bf = bayer_rg; break;
    case 2: bf = bayer_gb; break;
    case 3: bf = bayer_gr; break;
    case 4: bf = bayer_bg; break;
  }
  return bf;
}

binnings_t MVBinningToBinning(const TCameraBinningMode& idbm)
{
  binnings_t bin;
  switch (idbm)
  {
    default: bin = Bin_Off; break;  //*
    case cbmBinningH: bin = Bin_H; break;
    case cbmBinningV: bin = Bin_V; break;
    case cbmBinningHV: bin = Bin_HV; break;
    case cbmBinning3H: bin = Bin_3H; break;
    case cbmBinning3V: bin = Bin_3V; break;
    case cbmBinning3H3V: bin = Bin_3H3V; break;
    case cbmBinningHAvg: bin = Bin_HAvg; break;
    case cbmBinningVAvg: bin = Bin_VAvg; break;
    case cbmBinningHVAvg: bin = Bin_HVAvg; break;
    case cbmBinning3HAvg: bin = Bin_3HAvg; break;
    case cbmBinning3VAvg: bin = Bin_3VAvg; break;
    case cbmBinning3H3VAvg: bin = Bin_3H3VAvg; break;
    case cbmDroppingH: bin = Drop_H; break;
    case cbmDroppingV: bin = Drop_V; break;
    case cbmDroppingHV: bin = Drop_HV; break;
    case cbmDropping3H: bin = Drop_3H; break;
    case cbmDropping3V: bin = Drop_3V; break;
    case cbmDropping3H3V: bin = Drop_3H3V; break;
  }
  return bin;
}


TCameraBinningMode BinningToMVBinning(const binnings_t& bin)
{
  TCameraBinningMode idbm;
  switch (bin)
  {
    default: idbm = cbmOff; break;  //*
    case Bin_H: idbm = cbmBinningH; break;
    case Bin_V: idbm = cbmBinningV; break;
    case Bin_HV: idbm = cbmBinningHV; break;
    case Bin_3H: idbm = cbmBinning3H; break;
    case Bin_3V: idbm = cbmBinning3V; break;
    case Bin_3H3V: idbm = cbmBinning3H3V; break;
    case Bin_HAvg: idbm = cbmBinningHAvg; break;
    case Bin_VAvg: idbm = cbmBinningVAvg; break;
    case Bin_HVAvg: idbm = cbmBinningHVAvg; break;
    case Bin_3HAvg: idbm = cbmBinning3HAvg; break;
    case Bin_3VAvg: idbm = cbmBinning3VAvg; break;
    case Bin_3H3VAvg: idbm = cbmBinning3H3VAvg; break;
    case Drop_H: idbm = cbmDroppingH; break;
    case Drop_V: idbm = cbmDroppingV; break;
    case Drop_HV: idbm = cbmDroppingHV; break;
    case Drop_3H: idbm = cbmDropping3H; break;
    case Drop_3V: idbm = cbmDropping3V; break;
    case Drop_3H3V: idbm = cbmDropping3H3V; break;
  }
  return idbm;
}

} // End of CMvbluefox3 namespace
